/*
-----------------------------------------------------------------------------
This source file is part of OGRE
    (Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2005 The OGRE Team
Also see acknowledgements in Readme.html

You may use this sample code for anything you like, it is not covered by the
LGPL like the rest of the engine.
-----------------------------------------------------------------------------
*/
/*
-----------------------------------------------------------------------------
Filename:    ExampleFrameListener.h
Description: Defines an example frame listener which responds to frame events.
This frame listener just moves a specified camera around based on
keyboard and mouse movements.
Mouse:    Freelook
W or Up:  Forward
S or Down:Backward
A:        Step left
D:        Step right
             PgUp:     Move upwards
             PgDown:   Move downwards
             F:        Toggle frame rate stats on/off
			 R:        Render mode
             T:        Cycle texture filtering
                       Bilinear, Trilinear, Anisotropic(8)
             P:        Toggle on/off display of camera position / orientation
-----------------------------------------------------------------------------
*/

#ifndef __KdTreeAppListener_H__
#define __KdTreeAppListener_H__

#include "Ogre.h"
#include "OgreKeyEvent.h"
#include "OgreEventListeners.h"
#include "OgreStringConverter.h"
#include "OgreException.h"

#define VIZ_VIEWPORT_Z_ORDER 10

using namespace Ogre;

class KdTreeAppListener: public FrameListener, public KeyListener
{
protected:
	int mSceneDetailIndex ;
    Real mMoveSpeed;
    Degree mRotateSpeed;
	Real mRotationPeriod;
	bool mShowTree;
	bool mFreeMove;

	Real mDelayedQueriesIssued;
	Real mDelayedTraversedNodes;

	int mSeqNum;

	Overlay* mDebugOverlay;
	Overlay* mKdTreeOverlay;

	OverlayElement *mKdTreeMaxDepthInfo;
	OverlayElement *mKTInfo;
	OverlayElement *mKIInfo;
	OverlayElement *mHighlightLevelInfo;
	OverlayElement *mBuildMethodInfo;
	OverlayElement *mRenderMethodInfo;
	OverlayElement *mMovementInfo;

	OverlayElement *mFrustumCulledNodesInfo;
	OverlayElement *mQueryCulledNodesInfo;
	OverlayElement *mTraversedNodesInfo;
	OverlayElement *mHierarchyNodesInfo;
	OverlayElement *mRenderedNodesInfo;
	OverlayElement *mObjectsCountInfo;
	OverlayElement *mQueriesIssuedInfo;

	EventProcessor* mEventProcessor;
	InputReader* mInputDevice;
	SceneNode *mCamNode;
	Camera* mCamera;
	Camera* mTopCam;
	SceneManager *mSceneMgr;
	Radian mDeathAngle;

	Vector3 mTranslateVector;
	RenderWindow* mWindow;
	bool mStatsOn;
	bool mUseBufferedInputKeys, mUseBufferedInputMouse, mInputTypeSwitchingOn;
	unsigned int mNumScreenShots;
	float mMoveScale;
	Degree mRotScale;
	// just to stop toggles flipping too fast
	Real mTimeUntilNextToggle ;
	Radian mRotX, mRotY;
	TextureFilterOptions mFiltering;
	int mAniso;

	// stuff for the viz camera
	bool mVizCamera;
	Vector3 mVizCamTransVect;

    void updateStats(void);

public:

	// visualization modes for scene nodes
	enum 
	{
		NODEVIZ_NONE, 
		NODEVIZ_RENDER_NODES, 
		NODEVIZ_RENDER_NODES_AND_CONTENT, 
		NODEVIZ_MODES_NUM
	};

    // Constructor takes a RenderWindow because it uses that to determine input context
    KdTreeAppListener(RenderWindow* win, SceneManager* sm, Real rs, Real ms, Real rp,
		bool useBufferedInputKeys = false, bool useBufferedInputMouse = false);

    virtual ~KdTreeAppListener();

	void initOverlayElement(OverlayElement **elInfo, String ext, 
		String name, int top, String caption);

	void initStatsOverlay();

	void initKdTreeOverlay();

	void showDebugOverlay(bool show);

	void toggleVizCamera();

    virtual bool processUnbufferedKeyInput(const FrameEvent& evt);

    bool processUnbufferedMouseInput(const FrameEvent& evt);

	void moveCamera();

    // Override frameStarted event to process that (don't care about frameEnded)
    bool frameStarted(const FrameEvent& evt);

    bool frameEnded(const FrameEvent& evt);

	void switchMouseMode();

	void switchKeyMode();

	void keyClicked(KeyEvent* e);

	void keyPressed(KeyEvent* e) {};
	void keyReleased(KeyEvent* e) {};
};

#endif

